@@ -1,12 +1,10 @@
 // TarHandler.h
 
-#ifndef __TAR_HANDLER_H
-#define __TAR_HANDLER_H
+#ifndef ZIP7_INC_TAR_HANDLER_H
+#define ZIP7_INC_TAR_HANDLER_H
 
 #include "../../../Common/MyCom.h"
 
-#include "../../../Windows/PropVariant.h"
-
 #include "../../Compress/CopyCoder.h"
 
 #include "../Common/HandlerOut.h"
@@ -16,14 +14,12 @@
 namespace NArchive {
 namespace NTar {
 
-class CHandler:
-  public IInArchive,
-  public IArchiveOpenSeq,
-  public IInArchiveGetStream,
-  public ISetProperties,
-  public IOutArchive,
-  public CMyUnknownImp
-{
+Z7_CLASS_IMP_CHandler_IInArchive_4(
+    IArchiveOpenSeq
+  , IInArchiveGetStream
+  , ISetProperties
+  , IOutArchive
+)
 public:
   CObjectVector<CItemEx> _items;
   CMyComPtr<IInStream> _stream;
@@ -46,27 +42,12 @@ private:
 
   CArchive _arc;
 
-  NCompress::CCopyCoder *copyCoderSpec;
-  CMyComPtr<ICompressCoder> copyCoder;
+  CMyComPtr2_Create<ICompressCoder, NCompress::CCopyCoder> copyCoder;
 
   HRESULT Open2(IInStream *stream, IArchiveOpenCallback *callback);
   HRESULT SkipTo(UInt32 index);
   void TarStringToUnicode(const AString &s, NWindows::NCOM::CPropVariant &prop, bool toOs = false) const;
 public:
-  MY_UNKNOWN_IMP5(
-    IInArchive,
-    IArchiveOpenSeq,
-    IInArchiveGetStream,
-    ISetProperties,
-    IOutArchive
-  )
-
-  INTERFACE_IInArchive(;)
-  INTERFACE_IOutArchive(;)
-  STDMETHOD(OpenSeq)(ISequentialInStream *stream);
-  STDMETHOD(GetStream)(UInt32 index, ISequentialInStream **stream);
-  STDMETHOD(SetProperties)(const wchar_t * const *names, const PROPVARIANT *values, UInt32 numProps);
-
   void Init();
   CHandler();
 };
