@@ -19,8 +19,8 @@ static bool MakeAutoName(const FString &name,
 
 bool AutoRenamePath(FString &path)
 {
-  int dotPos = path.ReverseFind_Dot();
-  int slashPos = path.ReverseFind_PathSepar();
+  const int dotPos = path.ReverseFind_Dot();
+  const int slashPos = path.ReverseFind_PathSepar();
 
   FString name = path;
   FString extension;
@@ -29,14 +29,14 @@ bool AutoRenamePath(FString &path)
     name.DeleteFrom((unsigned)dotPos);
     extension = path.Ptr((unsigned)dotPos);
   }
-  name += '_';
+  name.Add_Char('_');
   
   FString temp;
 
-  UInt32 left = 1, right = ((UInt32)1 << 30);
+  UInt32 left = 1, right = (UInt32)1 << 30;
   while (left != right)
   {
-    UInt32 mid = (left + right) / 2;
+    const UInt32 mid = (left + right) / 2;
     if (MakeAutoName(name, extension, mid, temp))
       left = mid + 1;
     else
