@@ -2,8 +2,6 @@
 
 #include "StdAfx.h"
 
-#include "../../../../C/Sort.h"
-
 #include "../../../Common/StringConvert.h"
 
 #include "../../../Windows/FileDir.h"
@@ -43,6 +41,7 @@ static HRESULT DecompressArchive(
     const CExtractOptions &options,
     bool calcCrc,
     IExtractCallbackUI *callback,
+    IFolderArchiveExtractCallback *callbackFAE,
     CArchiveExtractCallback *ecs,
     UString &errorMessage,
     UInt64 &stdInProcessed)
@@ -94,7 +93,7 @@ static HRESULT DecompressArchive(
   if (!options.StdInMode)
   {
     UInt32 numItems;
-    RINOK(archive->GetNumberOfItems(&numItems));
+    RINOK(archive->GetNumberOfItems(&numItems))
     
     CReadArcItem item;
 
@@ -105,7 +104,7 @@ static HRESULT DecompressArchive(
           || options.ExcludeDirItems
           || options.ExcludeFileItems)
       {
-        RINOK(arc.GetItem(i, item));
+        RINOK(arc.GetItem(i, item))
         if (item.IsDir ? options.ExcludeDirItems : options.ExcludeFileItems)
           continue;
       }
@@ -115,7 +114,7 @@ static HRESULT DecompressArchive(
         item.IsAltStream = false;
         if (!options.NtOptions.AltStreams.Val && arc.Ask_AltStream)
         {
-          RINOK(Archive_IsItem_AltStream(arc.Archive, i, item.IsAltStream));
+          RINOK(Archive_IsItem_AltStream(arc.Archive, i, item.IsAltStream))
         }
         #endif
       }
@@ -194,12 +193,14 @@ static HRESULT DecompressArchive(
       options.NtOptions,
       options.StdInMode ? &wildcardCensor : NULL,
       &arc,
-      callback,
+      callbackFAE,
       options.StdOutMode, options.TestMode,
       outDir,
       removePathParts, false,
       packSize);
 
+  ecs->Is_elimPrefix_Mode = elimIsPossible;
+
   
   #ifdef SUPPORT_LINKS
   
@@ -207,14 +208,14 @@ static HRESULT DecompressArchive(
       !options.TestMode &&
       options.NtOptions.HardLinks.Val)
   {
-    RINOK(ecs->PrepareHardLinks(&realIndices));
+    RINOK(ecs->PrepareHardLinks(&realIndices))
   }
     
   #endif
 
   
   HRESULT result;
-  Int32 testMode = (options.TestMode && !calcCrc) ? 1: 0;
+  const Int32 testMode = (options.TestMode && !calcCrc) ? 1: 0;
 
   CArchiveExtractCallback_Closer ecsCloser(ecs);
 
@@ -226,9 +227,15 @@ static HRESULT DecompressArchive(
       ConvertPropVariantToUInt64(prop, stdInProcessed);
   }
   else
-    result = archive->Extract(&realIndices.Front(), realIndices.Size(), testMode, ecs);
+  {
+    // v23.02: we reset completed value that could be set by Open() operation
+    IArchiveExtractCallback *aec = ecs;
+    const UInt64 val = 0;
+    RINOK(aec->SetCompleted(&val))
+    result = archive->Extract(realIndices.ConstData(), realIndices.Size(), testMode, aec);
+  }
   
-  HRESULT res2 = ecsCloser.Close();
+  const HRESULT res2 = ecsCloser.Close();
   if (result == S_OK)
     result = res2;
 
@@ -270,7 +277,8 @@ HRESULT Extract(
     const CExtractOptions &options,
     IOpenCallbackUI *openCallback,
     IExtractCallbackUI *extractCallback,
-    #ifndef _SFX
+    IFolderArchiveExtractCallback *faeCallback,
+    #ifndef Z7_SFX
     IHashCalc *hash,
     #endif
     UString &errorMessage,
@@ -323,13 +331,13 @@ HRESULT Extract(
       options.ZoneMode,
       false // keepEmptyDirParts
       );
-  #ifndef _SFX
+  #ifndef Z7_SFX
   ecs->SetHashMethods(hash);
   #endif
 
   if (multi)
   {
-    RINOK(extractCallback->SetTotal(totalPackSize));
+    RINOK(faeCallback->SetTotal(totalPackSize))
   }
 
   UInt64 totalPackProcessed = 0;
@@ -347,11 +355,10 @@ HRESULT Extract(
     if (options.StdInMode)
     {
       // do we need ctime and mtime?
-      fi.ClearBase();
-      fi.Size = 0; // (UInt64)(Int64)-1;
-      fi.SetAsFile();
-      // NTime::GetCurUtc_FiTime(fi.MTime);
-      // fi.CTime = fi.ATime = fi.MTime;
+      // fi.ClearBase();
+      // fi.Size = 0; // (UInt64)(Int64)-1;
+      if (!fi.SetAs_StdInFile())
+        return GetLastError_noZero_HRESULT();
     }
     else
     {
@@ -364,17 +371,17 @@ HRESULT Extract(
     }
 
     /*
-    #ifndef _NO_CRYPTO
+    #ifndef Z7_NO_CRYPTO
     openCallback->Open_Clear_PasswordWasAsked_Flag();
     #endif
     */
 
-    RINOK(extractCallback->BeforeOpen(arcPath, options.TestMode));
+    RINOK(extractCallback->BeforeOpen(arcPath, options.TestMode))
     CArchiveLink arcLink;
 
     CObjectVector<COpenType> types2 = types;
     /*
-    #ifndef _SFX
+    #ifndef Z7_SFX
     if (types.IsEmpty())
     {
       int pos = arcPath.ReverseFind(L'.');
@@ -382,7 +389,7 @@ HRESULT Extract(
       {
         UString s = arcPath.Ptr(pos + 1);
         int index = codecs->FindFormatForExtension(s);
-        if (index >= 0 && s == L"001")
+        if (index >= 0 && s.IsEqualTo("001"))
         {
           s = arcPath.Left(pos);
           pos = s.ReverseFind(L'.');
@@ -402,7 +409,7 @@ HRESULT Extract(
     */
 
     COpenOptions op;
-    #ifndef _SFX
+    #ifndef Z7_SFX
     op.props = &options.Properties;
     #endif
     op.codecs = codecs;
@@ -418,7 +425,7 @@ HRESULT Extract(
       return result;
 
     // arcLink.Set_ErrorsText();
-    RINOK(extractCallback->OpenResult(codecs, arcLink, arcPath, result));
+    RINOK(extractCallback->OpenResult(codecs, arcLink, arcPath, result))
 
     if (result != S_OK)
     {
@@ -428,7 +435,7 @@ HRESULT Extract(
       continue;
     }
 
-   #if defined(_WIN32) && !defined(UNDER_CE) && !defined(_SFX)
+   #if defined(_WIN32) && !defined(UNDER_CE) && !defined(Z7_SFX)
     if (options.ZoneMode != NExtract::NZoneIdMode::kNone
         && !options.StdInMode)
     {
@@ -446,7 +453,13 @@ HRESULT Extract(
           /* real Extracting to files is possible.
              But user can think that hash archive contains real files.
              So we block extracting here. */
-          return E_NOTIMPL;
+          // v23.00 : we don't break process.
+          RINOK(extractCallback->OpenResult(codecs, arcLink, arcPath, E_NOTIMPL))
+          thereAreNotOpenArcs = true;
+          if (!options.StdInMode)
+            totalPackProcessed += fi.Size;
+          continue;
+          // return E_NOTIMPL; // before v23
         }
         FString dirPrefix = us2fs(options.HashDir);
         if (dirPrefix.IsEmpty())
@@ -490,7 +503,7 @@ HRESULT Extract(
           if (newPackSize < 0)
             newPackSize = 0;
           totalPackSize = (UInt64)newPackSize;
-          RINOK(extractCallback->SetTotal(totalPackSize));
+          RINOK(faeCallback->SetTotal(totalPackSize))
         }
       }
     }
@@ -498,13 +511,13 @@ HRESULT Extract(
     /*
     // Now openCallback and extractCallback use same object. So we don't need to send password.
 
-    #ifndef _NO_CRYPTO
+    #ifndef Z7_NO_CRYPTO
     bool passwordIsDefined;
     UString password;
-    RINOK(openCallback->Open_GetPasswordIfAny(passwordIsDefined, password));
+    RINOK(openCallback->Open_GetPasswordIfAny(passwordIsDefined, password))
     if (passwordIsDefined)
     {
-      RINOK(extractCallback->SetPassword(password));
+      RINOK(extractCallback->SetPassword(password))
     }
     #endif
     */
@@ -520,7 +533,7 @@ HRESULT Extract(
 
     UInt64 packProcessed;
     const bool calcCrc =
-        #ifndef _SFX
+        #ifndef Z7_SFX
           (hash != NULL);
         #else
           false;
@@ -533,7 +546,8 @@ HRESULT Extract(
         wildcardCensor,
         options,
         calcCrc,
-        extractCallback, ecs, errorMessage, packProcessed));
+        extractCallback, faeCallback, ecs,
+        errorMessage, packProcessed))
 
     if (!options.StdInMode)
       packProcessed = fi.Size + arcLink.VolumesSize;
@@ -546,8 +560,8 @@ HRESULT Extract(
 
   if (multi || thereAreNotOpenArcs)
   {
-    RINOK(extractCallback->SetTotal(totalPackSize));
-    RINOK(extractCallback->SetCompleted(&totalPackProcessed));
+    RINOK(faeCallback->SetTotal(totalPackSize))
+    RINOK(faeCallback->SetCompleted(&totalPackProcessed))
   }
 
   st.NumFolders = ecs->NumFolders;
