@@ -62,7 +62,7 @@ void MyStringUpper_Ascii(char *s) throw()
 {
   for (;;)
   {
-    char c = *s;
+    const char c = *s;
     if (c == 0)
       return;
     *s++ = MyCharUpper_Ascii(c);
@@ -73,7 +73,7 @@ void MyStringUpper_Ascii(wchar_t *s) throw()
 {
   for (;;)
   {
-    wchar_t c = *s;
+    const wchar_t c = *s;
     if (c == 0)
       return;
     *s++ = MyCharUpper_Ascii(c);
@@ -85,7 +85,7 @@ void MyStringLower_Ascii(char *s) throw()
 {
   for (;;)
   {
-    char c = *s;
+    const char c = *s;
     if (c == 0)
       return;
     *s++ = MyCharLower_Ascii(c);
@@ -96,7 +96,7 @@ void MyStringLower_Ascii(wchar_t *s) throw()
 {
   for (;;)
   {
-    wchar_t c = *s;
+    const wchar_t c = *s;
     if (c == 0)
       return;
     *s++ = MyCharLower_Ascii(c);
@@ -190,8 +190,8 @@ bool IsString1PrefixedByString2(const char *s1, const char *s2) throw()
 {
   for (;;)
   {
-    const unsigned char c2 = (unsigned char)*s2++; if (c2 == 0) return true;
-    const unsigned char c1 = (unsigned char)*s1++; if (c1 != c2) return false;
+    const char c2 = *s2++; if (c2 == 0) return true;
+    const char c1 = *s1++; if (c1 != c2) return false;
   }
 }
 
@@ -208,35 +208,6 @@ bool StringsAreEqualNoCase(const wchar_t *s1, const wchar_t *s2) throw()
 
 // ---------- ASCII ----------
 
-bool AString::IsPrefixedBy_Ascii_NoCase(const char *s) const throw()
-{
-  const char *s1 = _chars;
-  for (;;)
-  {
-    const char c2 = *s++;
-    if (c2 == 0)
-      return true;
-    const char c1 = *s1++;
-    if (MyCharLower_Ascii(c1) !=
-        MyCharLower_Ascii(c2))
-      return false;
-  }
-}
-
-bool UString::IsPrefixedBy_Ascii_NoCase(const char *s) const throw()
-{
-  const wchar_t *s1 = _chars;
-  for (;;)
-  {
-    const char c2 = *s++;
-    if (c2 == 0)
-      return true;
-    const wchar_t c1 = *s1++;
-    if (MyCharLower_Ascii(c1) != (unsigned char)MyCharLower_Ascii(c2))
-      return false;
-  }
-}
-
 bool StringsAreEqual_Ascii(const char *u, const char *a) throw()
 {
   for (;;)
@@ -402,26 +373,33 @@ void AString::InsertSpace(unsigned &index, unsigned size)
 }
 
 #define k_Alloc_Len_Limit (0x40000000 - 2)
+// #define k_Alloc_Len_Limit (((unsigned)1 << (sizeof(unsigned) * 8 - 2)) - 2)
 
 void AString::ReAlloc(unsigned newLimit)
 {
   // MY_STRING_REALLOC(_chars, char, (size_t)newLimit + 1, (size_t)_len + 1);
   char *newBuf = MY_STRING_NEW_char((size_t)newLimit + 1);
   memcpy(newBuf, _chars, (size_t)_len + 1);
-  MY_STRING_DELETE(_chars);
+  MY_STRING_DELETE(_chars)
   _chars = newBuf;
   _limit = newLimit;
 }
 
+#define THROW_STRING_ALLOC_EXCEPTION  { throw 20130220; }
+
+#define CHECK_STRING_ALLOC_LEN(len) \
+  { if ((len) > k_Alloc_Len_Limit) THROW_STRING_ALLOC_EXCEPTION }
+
 void AString::ReAlloc2(unsigned newLimit)
 {
-  if (newLimit > k_Alloc_Len_Limit) throw 20130220;
+  CHECK_STRING_ALLOC_LEN(newLimit)
   // MY_STRING_REALLOC(_chars, char, (size_t)newLimit + 1, 0);
   char *newBuf = MY_STRING_NEW_char((size_t)newLimit + 1);
   newBuf[0] = 0;
-  MY_STRING_DELETE(_chars);
+  MY_STRING_DELETE(_chars)
   _chars = newBuf;
   _limit = newLimit;
+  _len = 0;
 }
 
 void AString::SetStartLen(unsigned len)
@@ -432,6 +410,7 @@ void AString::SetStartLen(unsigned len)
   _limit = len;
 }
 
+Z7_NO_INLINE
 void AString::Grow_1()
 {
   unsigned next = _len;
@@ -442,7 +421,7 @@ void AString::Grow_1()
   if (next < _len || next > k_Alloc_Len_Limit)
     next = k_Alloc_Len_Limit;
   if (next <= _len)
-    throw 20130220;
+    THROW_STRING_ALLOC_EXCEPTION
   ReAlloc(next);
   // Grow(1);
 }
@@ -460,7 +439,7 @@ void AString::Grow(unsigned n)
   if (next < _len || next > k_Alloc_Len_Limit)
     next = k_Alloc_Len_Limit;
   if (next <= _len || next - _len < n)
-    throw 20130220;
+    THROW_STRING_ALLOC_EXCEPTION
   ReAlloc(next);
 }
 
@@ -541,7 +520,7 @@ AString &AString::operator=(char c)
   if (1 > _limit)
   {
     char *newBuf = MY_STRING_NEW_char(1 + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = 1;
   }
@@ -558,7 +537,7 @@ AString &AString::operator=(const char *s)
   if (len > _limit)
   {
     char *newBuf = MY_STRING_NEW_char((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -575,7 +554,7 @@ AString &AString::operator=(const AString &s)
   if (len > _limit)
   {
     char *newBuf = MY_STRING_NEW_char((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -600,7 +579,7 @@ void AString::SetFromWStr_if_Ascii(const wchar_t *s)
   if (len > _limit)
   {
     char *newBuf = MY_STRING_NEW_char((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -624,7 +603,7 @@ void AString::SetFromBstr_if_Ascii(BSTR s)
   if (len > _limit)
   {
     char *newBuf = MY_STRING_NEW_char((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -637,10 +616,14 @@ void AString::SetFromBstr_if_Ascii(BSTR s)
 }
 */
 
+void AString::Add_Char(char c) { operator+=(c); }
 void AString::Add_Space() { operator+=(' '); }
 void AString::Add_Space_if_NotEmpty() { if (!IsEmpty()) Add_Space(); }
 void AString::Add_LF() { operator+=('\n'); }
 void AString::Add_Slash() { operator+=('/'); }
+void AString::Add_Dot() { operator+=('.'); }
+void AString::Add_Minus() { operator+=('-'); }
+void AString::Add_Colon() { operator+=(':'); }
 
 AString &AString::operator+=(const char *s)
 {
@@ -693,8 +676,9 @@ void AString::SetFrom(const char *s, unsigned len) // no check
 {
   if (len > _limit)
   {
+    CHECK_STRING_ALLOC_LEN(len)
     char *newBuf = MY_STRING_NEW_char((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -704,6 +688,12 @@ void AString::SetFrom(const char *s, unsigned len) // no check
   _len = len;
 }
 
+void AString::SetFrom_Chars_SizeT(const char *s, size_t len)
+{
+  CHECK_STRING_ALLOC_LEN(len)
+  SetFrom(s, (unsigned)len);
+}
+
 void AString::SetFrom_CalcLen(const char *s, unsigned len) // no check
 {
   unsigned i;
@@ -766,7 +756,7 @@ int AString::ReverseFind_PathSepar() const throw()
   const char *p = _chars + _len - 1;
   for (;;)
   {
-    char c = *p;
+    const char c = *p;
     if (IS_PATH_SEPAR(c))
       return (int)(p - _chars);
     if (p == _chars)
@@ -903,8 +893,8 @@ void AString::Replace(const AString &oldString, const AString &newString)
     return; // 0;
   if (oldString == newString)
     return; // 0;
-  unsigned oldLen = oldString.Len();
-  unsigned newLen = newString.Len();
+  const unsigned oldLen = oldString.Len();
+  const unsigned newLen = newString.Len();
   // unsigned number = 0;
   int pos = 0;
   while ((unsigned)pos < _len)
@@ -1001,30 +991,32 @@ void UString::ReAlloc(unsigned newLimit)
   // MY_STRING_REALLOC(_chars, wchar_t, (size_t)newLimit + 1, (size_t)_len + 1);
   wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)newLimit + 1);
   wmemcpy(newBuf, _chars, _len + 1);
-  MY_STRING_DELETE(_chars);
+  MY_STRING_DELETE(_chars)
   _chars = newBuf;
   _limit = newLimit;
 }
 
 void UString::ReAlloc2(unsigned newLimit)
 {
-  if (newLimit > k_Alloc_Len_Limit) throw 20130221;
+  CHECK_STRING_ALLOC_LEN(newLimit)
   // MY_STRING_REALLOC(_chars, wchar_t, newLimit + 1, 0);
   wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)newLimit + 1);
   newBuf[0] = 0;
-  MY_STRING_DELETE(_chars);
+  MY_STRING_DELETE(_chars)
   _chars = newBuf;
   _limit = newLimit;
+  _len = 0;
 }
 
 void UString::SetStartLen(unsigned len)
 {
-  _chars = 0;
+  _chars = NULL;
   _chars = MY_STRING_NEW_wchar_t((size_t)len + 1);
   _len = len;
   _limit = len;
 }
 
+Z7_NO_INLINE
 void UString::Grow_1()
 {
   unsigned next = _len;
@@ -1035,7 +1027,7 @@ void UString::Grow_1()
   if (next < _len || next > k_Alloc_Len_Limit)
     next = k_Alloc_Len_Limit;
   if (next <= _len)
-    throw 20130220;
+    THROW_STRING_ALLOC_EXCEPTION
   ReAlloc(next);
 }
 
@@ -1052,7 +1044,7 @@ void UString::Grow(unsigned n)
   if (next < _len || next > k_Alloc_Len_Limit)
     next = k_Alloc_Len_Limit;
   if (next <= _len || next - _len < n)
-    throw 20130220;
+    THROW_STRING_ALLOC_EXCEPTION
   ReAlloc(next - 1);
 }
 
@@ -1101,7 +1093,7 @@ UString operator+(const wchar_t *s1, const UString &s2) { return UString(s1, MyS
 
 UString::UString()
 {
-  _chars = 0;
+  _chars = NULL;
   _chars = MY_STRING_NEW_wchar_t(kStartStringCapacity);
   _len = 0;
   _limit = kStartStringCapacity - 1;
@@ -1163,7 +1155,7 @@ UString &UString::operator=(wchar_t c)
   if (1 > _limit)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t(1 + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = 1;
   }
@@ -1180,7 +1172,7 @@ UString &UString::operator=(const wchar_t *s)
   if (len > _limit)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -1197,7 +1189,7 @@ UString &UString::operator=(const UString &s)
   if (len > _limit)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -1210,8 +1202,9 @@ void UString::SetFrom(const wchar_t *s, unsigned len) // no check
 {
   if (len > _limit)
   {
+    CHECK_STRING_ALLOC_LEN(len)
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -1234,7 +1227,7 @@ void UString::SetFromBstr(LPCOLESTR s)
     if (c >= 0xd800 && c < 0xdc00 && i + 1 != len)
     {
       wchar_t c2 = s[i];
-      if (c2 >= 0xdc00 && c2 < 0x10000)
+      if (c2 >= 0xdc00 && c2 < 0xe000)
       {
         c = 0x10000 + ((c & 0x3ff) << 10) + (c2 & 0x3ff);
         i++;
@@ -1249,7 +1242,7 @@ void UString::SetFromBstr(LPCOLESTR s)
   if (len > _limit)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -1265,7 +1258,7 @@ void UString::SetFromBstr(LPCOLESTR s)
     if (c >= 0xd800 && c < 0xdc00 && i + 1 != len)
     {
       wchar_t c2 = *s;
-      if (c2 >= 0xdc00 && c2 < 0x10000)
+      if (c2 >= 0xdc00 && c2 < 0xe000)
       {
         s++;
         c = 0x10000 + ((c & 0x3ff) << 10) + (c2 & 0x3ff);
@@ -1289,7 +1282,7 @@ UString &UString::operator=(const char *s)
   if (len > _limit)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = newBuf;
     _limit = len;
   }
@@ -1301,20 +1294,15 @@ UString &UString::operator=(const char *s)
   return *this;
 }
 
+void UString::Add_Char(char c) { operator+=((wchar_t)(unsigned char)c); }
+// void UString::Add_WChar(wchar_t c) { operator+=(c); }
+void UString::Add_Dot() { operator+=(L'.'); }
 void UString::Add_Space() { operator+=(L' '); }
+void UString::Add_Minus() { operator+=(L'-'); }
+void UString::Add_Colon() { operator+=(L':'); }
+void UString::Add_LF() { operator+=(L'\n'); }
 void UString::Add_Space_if_NotEmpty() { if (!IsEmpty()) Add_Space(); }
 
-void UString::Add_LF()
-{
-  if (_limit == _len)
-    Grow_1();
-  unsigned len = _len;
-  wchar_t *chars = _chars;
-  chars[len++] = L'\n';
-  chars[len] = 0;
-  _len = len;
-}
-
 UString &UString::operator+=(const wchar_t *s)
 {
   unsigned len = MyStringLen(s);
@@ -1393,31 +1381,26 @@ int UString::ReverseFind(wchar_t c) const throw()
 {
   if (_len == 0)
     return -1;
-  const wchar_t *p = _chars + _len - 1;
-  for (;;)
+  const wchar_t *p = _chars + _len;
+  do
   {
-    if (*p == c)
+    if (*(--p) == c)
       return (int)(p - _chars);
-    if (p == _chars)
-      return -1;
-    p--;
   }
+  while (p != _chars);
+  return -1;
 }
 
 int UString::ReverseFind_PathSepar() const throw()
 {
-  if (_len == 0)
-    return -1;
-  const wchar_t *p = _chars + _len - 1;
-  for (;;)
+  const wchar_t *p = _chars + _len;
+  while (p != _chars)
   {
-    wchar_t c = *p;
+    const wchar_t c = *(--p);
     if (IS_PATH_SEPAR(c))
       return (int)(p - _chars);
-    if (p == _chars)
-      return -1;
-    p--;
   }
+  return -1;
 }
 
 void UString::TrimLeft() throw()
@@ -1597,11 +1580,11 @@ void UString::DeleteFrontal(unsigned num) throw()
 void UString2::ReAlloc2(unsigned newLimit)
 {
   // wrong (_len) is allowed after this function
-  if (newLimit > k_Alloc_Len_Limit) throw 20130221;
+  CHECK_STRING_ALLOC_LEN(newLimit)
   // MY_STRING_REALLOC(_chars, wchar_t, newLimit + 1, 0);
   if (_chars)
   {
-    MY_STRING_DELETE(_chars);
+    MY_STRING_DELETE(_chars)
     _chars = NULL;
     // _len = 0;
   }
@@ -1651,7 +1634,7 @@ UString2 &UString2::operator=(wchar_t c)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t(1 + 1);
     if (_chars)
-      MY_STRING_DELETE(_chars);
+      MY_STRING_DELETE(_chars)
     _chars = newBuf;
   }
   _len = 1;
@@ -1669,7 +1652,7 @@ UString2 &UString2::operator=(const wchar_t *s)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
     if (_chars)
-      MY_STRING_DELETE(_chars);
+      MY_STRING_DELETE(_chars)
     _chars = newBuf;
   }
   _len = len;
@@ -1684,7 +1667,7 @@ void UString2::SetFromAscii(const char *s)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
     if (_chars)
-      MY_STRING_DELETE(_chars);
+      MY_STRING_DELETE(_chars)
     _chars = newBuf;
   }
   wchar_t *chars = _chars;
@@ -1703,7 +1686,7 @@ UString2 &UString2::operator=(const UString2 &s)
   {
     wchar_t *newBuf = MY_STRING_NEW_wchar_t((size_t)len + 1);
     if (_chars)
-      MY_STRING_DELETE(_chars);
+      MY_STRING_DELETE(_chars)
     _chars = newBuf;
   }
   _len = len;
@@ -1793,3 +1776,67 @@ FString us2fs(const wchar_t *s)
 }
 
 #endif // USE_UNICODE_FSTRING
+
+
+bool CStringFinder::FindWord_In_LowCaseAsciiList_NoCase(const char *p, const wchar_t *str)
+{
+  _temp.Empty();
+  for (;;)
+  {
+    const wchar_t c = *str++;
+    if (c == 0)
+      break;
+    if (c <= 0x20 || c > 0x7f)
+      return false;
+    _temp.Add_Char((char)MyCharLower_Ascii((char)c));
+  }
+
+  while (*p != 0)
+  {
+    const char *s2 = _temp.Ptr();
+    char c, c2;
+    do
+    {
+      c = *p++;
+      c2 = *s2++;
+    }
+    while (c == c2);
+    
+    if (c == ' ')
+    {
+      if (c2 == 0)
+        return true;
+      continue;
+    }
+    
+    while (*p++ != ' ');
+  }
+  
+  return false;
+}
+
+
+void SplitString(const UString &srcString, UStringVector &destStrings)
+{
+  destStrings.Clear();
+  unsigned len = srcString.Len();
+  if (len == 0)
+    return;
+  UString s;
+  for (unsigned i = 0; i < len; i++)
+  {
+    const wchar_t c = srcString[i];
+    if (c == ' ')
+    {
+      if (!s.IsEmpty())
+      {
+        destStrings.Add(s);
+        s.Empty();
+      }
+    }
+    else
+      s += c;
+  }
+  if (!s.IsEmpty())
+    destStrings.Add(s);
+}
