@@ -1,7 +1,7 @@
 // Common/StdInStream.h
 
-#ifndef __COMMON_STD_IN_STREAM_H
-#define __COMMON_STD_IN_STREAM_H
+#ifndef ZIP7_INC_COMMON_STD_IN_STREAM_H
+#define ZIP7_INC_COMMON_STD_IN_STREAM_H
 
 #include <stdio.h>
 
@@ -11,20 +11,22 @@
 class CStdInStream
 {
   FILE *_stream;
-  bool _streamIsOpen;
+  // bool _streamIsOpen;
 public:
   int CodePage;
 
   CStdInStream(FILE *stream = NULL):
       _stream(stream),
-      _streamIsOpen(false),
+      // _streamIsOpen(false),
       CodePage(-1)
-      {};
+      {}
 
+  /*
   ~CStdInStream() { Close(); }
 
   bool Open(LPCTSTR fileName) throw();
   bool Close() throw();
+  */
 
   // returns:
   //   false, if ZERO character in stream
